﻿/*! @mainpage StovePCSDK
*/

/**
* @file StovePCSDK.h 
* @brief StovePCSDK 헤더파일
* @details StovePCSDK에서 제공하는 함수를 선언합니다.
*/
#ifndef STOVEPCSDK_H_
#define STOVEPCSDK_H_

#include "StovePCDefine.h"
#include "StovePCCallback.h"

#ifdef STOVEPCSDK_EXPORTS
#define STOVEPCSDK_API __declspec(dllexport)
#else
#define STOVEPCSDK_API __declspec(dllimport)
#endif


extern "C"
{
    /*!
    *  @fn StovePCResult StovePC_Init(const StovePCConfig& config, const StovePCCallback& callback)
    *  @brief StovePCSDK를 초기화 합니다.
    *  @param[in] StovePCConfig StovePCSDK를 초기화를 위한 설정 정보
    *  @param[in] StovePCCallback StovePCSDK 함수 호출 후 응답받는 콜백 구조체
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_Init(const StovePCConfig& config, const StovePCCallback& callback);

    /*!
    *  @fn StovePCResult StovePC_IAPInit(const char* shopKey)
    *  @brief IAP 를 초기화 합니다.
    *  @param[in] const char* shopKey 스토브에서 발급받은 상점 Key 빌링서비스 사용시 필수 입력
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_IAPInit(const char* shopKey);

    /*!
    *  @fn StovePCResult StovePC_UnInit()
    *  @brief StovePCSDK의 자원을 해제하고, 종료합니다. 
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_UnInit();

    /*!
    *  @fn StovePCResult StovePC_GetInitState()
    *  @brief StovePC_Init을 호출한 후의 PC SDK 내부 상태를 반환합니다.
    *  @return enum StovePCInitState PC SDK의 초기화 상태 값
    */
    STOVEPCSDK_API StovePCInitState StovePC_GetInitState();


    /*!
    *  @fn StovePCResult StovePC_RunCallback()
    *  @brief 등록한 콜백을 호출합니다.
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_RunCallback();

    /*!
    *  @fn StovePCResult StovePC_GetToken()
    *  @brief 로그인한 사용자의 토큰을 조회합니다. 함수 호출이 성공하면 OnToken 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과   
    */
    STOVEPCSDK_API StovePCResult StovePC_GetToken();

    /*!
    *  @fn StovePCResult StovePC_GetUser()
    *  @brief 로그인한 사용자의 정보를 조회합니다. 함수 호출이 성공하면 OnUser 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetUser();

    /*!
    *  @fn StovePCResult StovePC_GetOwnership()
    *  @brief StovePCConfig에 설정한 게임 아이디의 소유권을 조회합니다. 함수 호출이 성공하면 OnOwnership 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetOwnership();

    /*!
    *  @fn StovePCResult StovePC_GetStat(const char* statId)
    *  @brief 로그인한 사용자의 스탯을 조회합니다. 함수 호출이 성공하면 OnStat 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetStat(const char* statId);

    /*!
    *  @fn StovePCResult StovePC_SetStat(const char* statId, const int statValue)
    *  @brief 로그인한 사용자의 스탯을 설정합니다. 함수 호출이 성공하면 OnSetStat 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_SetStat(const char* statId, const int statValue);

    /*!
    *  @fn StovePCResult StovePC_GetAchievement(const char* achievementId)
    *  @brief 로그인한 사용자의 단일 업적을 조회합니다. 함수 호출이 성공하면 OnAchievement 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetAchievement(const char* achievementId);

    /*!
    *  @fn StovePCResult StovePC_GetAllAchievement()
    *  @brief 로그인한 사용자의 전체 업적을 조회합니다. 함수 호출이 성공하면 OnAllAchievement 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetAllAchievement();

    /*!
    *  @fn StovePCResult StovePCResult StovePC_GetRank(const char* leaderboardId, const unsigned int pageIndex, const unsigned int pageSize, bool includeMyRank)
    *  @brief 특정 페이지의 랭킹을 조회합니다. 함수 호출이 성공하면 OnRank 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetRank(const char* leaderboardId, const unsigned int pageIndex, const unsigned int pageSize, bool includeMyRank);

    /*!
    *  @fn StovePCResult StovePC_SetGameProfile(const char* worldId, const int64_t characterNo)
    *  @brief 게임프로필을 설정합니다. 별도의 콜백호출은 없습니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_SetGameProfile(const char* worldId, const int64_t characterNo);

    /*!
    *  @fn StovePCResult StovePC_GetAutoPopup()
    *  @brief 자동 팝업 정보를 조회합니다. 함수 호출이 성공하면 OnAutoPopup 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetAutoPopup();

    /*!
    *  @fn StovePCResult StovePC_GetManualPopup(const char* resourceKey)
    *  @brief 수동 팝업 정보를 조회합니다. 함수 호출이 성공하면 OnManualPopup 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetManualPopup(const char* resourceKey);

    /*!
    *  @fn StovePCResult StovePC_GetNewsPopup()
    *  @brief 뉴스 팝업 정보를 조회합니다. 함수 호출이 성공하면 OnNewsPopup 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetNewsPopup();

    /*!
    *  @fn StovePCResult StovePC_GetCouponPopup()
    *  @brief 쿠폰 팝업 정보를 조회합니다. 함수 호출이 성공하면 OnCouponPopup 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetCouponPopup();

    /*!
    *  @fn StovePCResult StovePC_GetCommunityPopup();
    *  @brief 커뮤니티 팝업 정보를 조회합니다. 함수 호출이 성공하면 OnCommunityPopup 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_GetCommunityPopup();


    /*!
    *  @fn StovePCResult StovePC_SetPopupDisallowed(const int popupId, const unsigned int disallowDays)
    *  @brief 팝업이 일정기간 동안 보여지지 않도록 설정합니다.
    *  @return struct StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_SetPopupDisallowed(const int popupId, const int days);

    /*!
    *  @fn char* StovePC_GetSDKVersion()
    *  @brief StovePCSDK의 버전 정보를 반환합니다.
    *  @return char* StovePCSDK의 Version 정보
    */
    STOVEPCSDK_API char* StovePC_GetSDKVersion();

    /*!
    *  @fn void StovePC_SetLanguage(const char* language)
    *  @brief StovePCSDK의 언어 정보를 설정합니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_SetLanguage(const char* language);

    /*!
    *  @fn void StovePC_TranslateLanguage(const char* stringId)
    *  @brief StovePCSDK에 설정된 언어 정보를 참고하여 번역합니다.
    *  @return wchar_t* 번역된 문자열
    */
    STOVEPCSDK_API wchar_t* StovePC_TranslateLanguage(const char* stringId);

    /*!
    *  @fn StovePCResult StovePC_StashCustomEvent(const char* name, const char* category1, const char* category2, float simpleValue, const StovePCCustomEventParameter* params, int paramsSize)
    *  @brief 이벤트(사건)를 전송합니다. 함수 호출이 성공하면 OnSendEvent 콜백이 호출됩니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_StashCustomEvent(const char* name, const char* category1, const char* category2, float simpleValue, const StovePCCustomEventParameter* params, int paramsSize);

    /*!
    *  @fn StovePCTraceHint StovePC_GetTraceHint()
    *  @brief 플랫폼 동선 추적을 위한 정보를 조회합니다.
    *  @return struct StovePCTraceHint 세션아이디 및 동선 추적을 위한 부가정보를 포함한 구조제
    */
    STOVEPCSDK_API StovePCTraceHint StovePC_GetTraceHint();

    /*!
    *  @fn StovePCResult StovePC_FetchShopCategories()
    *  @brief 스토브 플랫폼에 등록된 상점 카테코리를 조회합니다. 
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_FetchShopCategories();

    /*!
    *  @fn StovePCResult StovePC_FetchProducts()
    *  @brief 스토브 플랫폼에 등록된 상품 정보를 조회합니다. 
    *  @param[in] categoryId 상점카테고리 아이디 ("": 전체 상품목록, "061A30EC8C904CA" : 카테고리 Key)
    *  @param[in] isRefersh (true: 서버에서 조회 합니다. false: 캐시에서 조회 합니다)
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_FetchProducts(const char* categoryId, const bool isRefresh);

    /*!
    *  @fn StovePCResult StovePC_StartPurchase()
    *  @brief 상품구매를 시작하기 위한 결제 URL 을 전달 받습니다. 
    *  @param[in] products 주문 상품정보 구조체
    *  @param[in] productSize 주문 상품 구조체 갯수
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_StartPurchase(const StovePCOrderProduct* products, int productsSize);

    /*!
    *  @fn StovePCResult StovePC_ConfirmPurchase()
    *  @brief 상품구매 완료후 구매 ID로 정상처리 되었는지 확인 합니다.
    *  @param[in] transactionMasterNo 주문 마스터 번호
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_ConfirmPurchase(const int64_t transactionMasterNo);

    /*!
    *  @fn StovePCResult StovePC_VerifyPurchase()
    *  @brief 상품 카테고리의 등록된 상품ID로 구매여부를 조회 합니다. 
    *  @param[in] categoryId 카테고리 아이디 ( "" : 전체에서 조회, "061A30EC8C904CA" : 카테고리 Key) 
    *  @param[in] purchaseId 상품 아이디
    *  @return enum StovePCPurchaseVerification 상품구매확인 구조체 
    */
    STOVEPCSDK_API StovePCPurchaseVerification StovePC_VerifyPurchase(const char* categoryId, const int64_t productId);

    /*!
    *  @fn StovePCResult StovePC_FetchInventory()
    *  @brief 전체 구매한 상품목록을 조회합니다.
    *  @return enum StovePCResult 함수 호출 결과
    */
    STOVEPCSDK_API StovePCResult StovePC_FetchInventory();
}
/*#endif*/

#endif //STOVEPCSDK_H_
