﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "StoveSDKObject.h"
#include "StoveSDKNativeCallback.h"
#include "RunTime/CoreUObject/Public/UObject/UObjectIterator.h"
#include "EngineUtils.h"
#include "Engine/Engine.h"
#include "Interfaces/IPluginManager.h"
#include "StoveSDKPopup.h"

#include <string>

#define SDK_OBJECT_LOG(logContent) UE_LOG(LogStoveSDKPlugin, Log, TEXT(logContent));
#define SDK_OBJECT_LOG_FUNC(logContent) UE_LOG(LogStoveSDKPlugin, Log, TEXT(logContent), *FString(__func__));
#define SDK_OBJECT_LOG_APINAME(logContent, apiName) UE_LOG(LogStoveSDKPlugin, Log, TEXT(logContent), *apiName);

#define CHECK_POPUP_URL(Url) if (Url.IsEmpty()) { SDK_OBJECT_LOG_FUNC("%s: popup URL is empty"); return; }
#define CHECK_POPUP_LIST_EMPTY(popups) if (popups.Num() <= 0) { SDK_OBJECT_LOG_FUNC("%s: popup data is empty"); return; }
#define CHECK_HEADER_EMPTY() if (Headers.Num() <= 0) { SDK_OBJECT_LOG_FUNC("%s: popup header is empty"); return; }
#define CHECK_COOKIE_EMPTY() if (Cookies.Num() <= 0) { SDK_OBJECT_LOG_FUNC("%s: popup cookie is empty"); return; }
#define CHECK_MAKE_POPUP() if (!_MakePopupWidget(__func__)) return;

UStoveSDKObject::UStoveSDKObject()
{

}

void UStoveSDKObject::Tick(float DeltaTime)
{
    if (StoveSDKGetInitState().State == STOVE_PC_INIT_PENDING ||
        StoveSDKGetInitState().State == STOVE_PC_INIT_COMPLETE)
    {
        StovePCResult result = GetStoveSDKModule().RunCallback();
    }
}

bool UStoveSDKObject::IsTickable() const
{
    return true;
}

TStatId UStoveSDKObject::GetStatId() const
{
    return TStatId();
}

StovePCCallback preSettingCallback
{
    // basic
    ::OnError,
    ::OnInitComplete,
    ::OnToken,
    ::OnUser,
    ::OnOwnership,
    // stat/achieve/rank
    ::OnStat,
    ::OnSetStat,
    ::OnAchievement,
    ::OnAllAchievement,
    ::OnRank,
    // popup
    ::OnAutoPopup,
    ::OnManualPopup,
    ::OnNewsPopup,
    ::OnCouponPopup,
    ::OnCommunityPopup,
    // additonal
    ::OnStashCustomEvent,
    ::OnRenewToken,
    ::OnOverImmersion,
    ::OnShutdown,
    // billing
    ::OnFetchShopCategories,
    ::OnFetchProducts,
    ::OnStartPurchase,
    ::OnConfirmPurchase,
    ::OnFetchInventory
};

void UStoveSDKObject::SetCheckBoxState(const TArray<ECheckBoxState>& CallbackCheck)
{
#pragma region CallbackCheck
    if (CallbackCheck[0] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnError = nullptr;
    }
    else
    {
        preSettingCallback.OnError = ::OnError;
    }

    if (CallbackCheck[1] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnInitComplete = nullptr;
    }
    else
    {
        preSettingCallback.OnInitComplete = ::OnInitComplete;
    }

    if (CallbackCheck[2] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnToken = nullptr;
    }
    else
    {
        preSettingCallback.OnToken = ::OnToken;
    }

    if (CallbackCheck[3] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnUser = nullptr;
    }
    else
    {
        preSettingCallback.OnUser = ::OnUser;
    }

    if (CallbackCheck[4] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnOwnership = nullptr;
    }
    else
    {
        preSettingCallback.OnOwnership = ::OnOwnership;
    }

    // GameSupport
    if (CallbackCheck[5] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnStat = nullptr;
    }
    else
    {
        preSettingCallback.OnStat = ::OnStat;
    }
    if (CallbackCheck[6] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnSetStat = nullptr;
    }
    else
    {
        preSettingCallback.OnSetStat = ::OnSetStat;
    }

    if (CallbackCheck[7] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnAchievement = nullptr;
    }
    else
    {
        preSettingCallback.OnAchievement = ::OnAchievement;
    }
    if (CallbackCheck[8] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnAllAchievement = nullptr;
    }
    else
    {
        preSettingCallback.OnAllAchievement = ::OnAllAchievement;
    }
    if (CallbackCheck[9] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnRank = nullptr;
    }
    else
    {
        preSettingCallback.OnRank = ::OnRank;
    }

    // Popup
    if (CallbackCheck[10] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnAutoPopup = nullptr;
    }
    else
    {
        preSettingCallback.OnAutoPopup = ::OnAutoPopup;
    }

    if (CallbackCheck[11] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnManualPopup = nullptr;
    }
    else
    {
        preSettingCallback.OnManualPopup = ::OnManualPopup;
    }

    if (CallbackCheck[12] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnNewsPopup = nullptr;
    }
    else
    {
        preSettingCallback.OnNewsPopup = ::OnNewsPopup;
    }

    if (CallbackCheck[13] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnCouponPopup = nullptr;
    }
    else
    {
        preSettingCallback.OnCouponPopup = ::OnCouponPopup;
    }

    if (CallbackCheck[14] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnCommunityPopup = nullptr;
    }
    else
    {
        preSettingCallback.OnCommunityPopup = ::OnCommunityPopup;
    }

    //etc 
    if (CallbackCheck[15] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnStashCustomEvent = nullptr;
    }
    else
    {
        preSettingCallback.OnStashCustomEvent = ::OnStashCustomEvent;
    }
    if (CallbackCheck[16] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnRenewToken = nullptr;
    }
    else
    {
        preSettingCallback.OnRenewToken = ::OnRenewToken;
    }
    if (CallbackCheck[17] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnOverImmersion = nullptr;
    }
    else
    {
        preSettingCallback.OnOverImmersion = ::OnOverImmersion;
    }
    if (CallbackCheck[18] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnShutdown = nullptr;
    }
    else
    {
        preSettingCallback.OnShutdown = ::OnShutdown;
    }
    // billing
    if (CallbackCheck[19] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnFetchShopCategories = nullptr;
    }
    else
    {
        preSettingCallback.OnFetchShopCategories = ::OnFetchShopCategories;
    }
    if (CallbackCheck[20] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnFetchProducts = nullptr;
    }
    else
    {
        preSettingCallback.OnFetchProducts = ::OnFetchProducts;
    }
    if (CallbackCheck[21] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnStartPurchase = nullptr;
    }
    else
    {
        preSettingCallback.OnStartPurchase = ::OnStartPurchase;
    }
    if (CallbackCheck[22] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnConfirmPurchase = nullptr;
    }
    else
    {
        preSettingCallback.OnConfirmPurchase = ::OnConfirmPurchase;
    }
    if (CallbackCheck[23] == ECheckBoxState::Unchecked)
    {
        preSettingCallback.OnFetchInventory = nullptr;
    }
    else
    {
        preSettingCallback.OnFetchInventory = ::OnFetchInventory;
    }
#pragma endregion CallbackCheck

}

void UStoveSDKObject::BeginDestroy()
{
    StovePCResult result = GetStoveSDKModule().UnInit();
    if (result == STOVE_PC_NO_ERROR)
    {
        SDK_OBJECT_LOG("StovePC_RunCallback --> Stop");
    }

    Super::BeginDestroy();
}

FStoveResult UStoveSDKObject::StoveSDKInit(const FStoveConfig& Config)
{
    StovePCConfig config;
    memset(&config, 0, sizeof(StovePCConfig));

    std::string nativeEnv = StringCast<ANSICHAR>(*Config.Env).Get();
    std::string nativeAppKey = StringCast<ANSICHAR>(*Config.AppKey).Get();
    std::string nativeSecret = StringCast<ANSICHAR>(*Config.SecretKey).Get();
    std::wstring nativeGameId = StringCast<WIDECHAR>(*Config.GameId).Get();
    std::wstring nativeLogPath = StringCast<WIDECHAR>(*Config.LogPath).Get();

    config.env = const_cast<char*>(nativeEnv.c_str());
    config.appKey = const_cast<char*>(nativeAppKey.c_str());
    config.secretKey = const_cast<char*>(nativeSecret.c_str());
    config.gameId = const_cast<wchar_t*>(nativeGameId.c_str());
    config.logLevel = StovePCLogLevel(Config.LogLevel);
    config.logPath = const_cast<wchar_t*>(nativeLogPath.c_str());

    StovePCResult result = GetStoveSDKModule().Init(config, preSettingCallback);
    if (result == STOVE_PC_NO_ERROR)
    {
        SDK_OBJECT_LOG("StovePC_RunCallback --> Start");
    }

    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKIAPInit(const FString& ShopKey)
{
    std::string shopKey = StringCast<ANSICHAR>(*ShopKey).Get();
    StovePCResult result = GetStoveSDKModule().IAPInit(shopKey.c_str());
    return FStoveResult{ result };
}

FStoveState UStoveSDKObject::StoveSDKGetInitState()
{
    StovePCInitState state = GetStoveSDKModule().InitState();
    return FStoveState{ state };
}


FStoveResult UStoveSDKObject::StoveSDKGetToken()
{
    StovePCResult result = GetStoveSDKModule().GetToken();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetUser()
{
    StovePCResult result = GetStoveSDKModule().GetUser();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetOwnership()
{
    StovePCResult result = GetStoveSDKModule().GetOwnership();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetStat(const FString& StatId)
{
    std::string statId = StringCast<ANSICHAR>(*StatId).Get();
    StovePCResult result = GetStoveSDKModule().GetStat(statId.c_str());
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKSetStat(const FString& StatId, const int StatValue)
{
    std::string statId = StringCast<ANSICHAR>(*StatId).Get();
    StovePCResult Result = GetStoveSDKModule().SetStat(statId.c_str(), StatValue);
    return FStoveResult{ Result };
}

FStoveResult UStoveSDKObject::StoveSDKGetAchievement(const FString& AchievementId)
{
    std::string achievementId = StringCast<ANSICHAR>(*AchievementId).Get();
    StovePCResult result = GetStoveSDKModule().GetAchievement(achievementId.c_str());
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetAllAchievement()
{
    StovePCResult result = GetStoveSDKModule().GetAllAchievement();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetRank(const FString& LeaderboardId, const int PageIndex, const int PageSize, bool IncludeMyRank)
{
    std::string leaderboardId = StringCast<ANSICHAR>(*LeaderboardId).Get();
    StovePCResult result = GetStoveSDKModule().GetRank(leaderboardId.c_str(), PageIndex, PageSize, IncludeMyRank);
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKUnInit()
{
    StovePCResult result = GetStoveSDKModule().UnInit();
    if (result == STOVE_PC_NO_ERROR)
    {
        SDK_OBJECT_LOG("StovePC_RunCallback --> Stop");
    }
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKSetGameProfile(const FString& WorldId, const FString& CharactorNo)
{
    std::string worldId = StringCast<ANSICHAR>(*WorldId).Get();
    int64 charactorNo = FCString::Strtoi64(*CharactorNo, NULL, 10);

    StovePCResult result = GetStoveSDKModule().SetGameProfile(worldId.c_str(), charactorNo);
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetAutoPopup()
{
    StovePCResult result = GetStoveSDKModule().GetAutoPopup();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetManualPopup(const FString & ResourceKey)
{
    std::string resourceKey = StringCast<ANSICHAR>(*ResourceKey).Get();
    StovePCResult result = GetStoveSDKModule().GetManualPopup(resourceKey.c_str());
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetNewsPopup()
{
    StovePCResult result = GetStoveSDKModule().GetNewsPopup();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetCouponPopup()
{
    StovePCResult result = GetStoveSDKModule().GetCouponPopup();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKGetCommunityPopup()
{
    StovePCResult result = GetStoveSDKModule().GetCommunityPopup();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKStashCustomEvent(const FString& Name,
                                                       const FString& Category1,
                                                       const FString& Category2,
                                                       const FString& SimpleValue,
                                                       const TArray<FStoveCustomEventParameter> Params,
                                                       int ParamSize)
{
    if (Params.Num() <= 0 ||
        Params.Num() != ParamSize ||
        ParamSize <= 0)
    {
        return FStoveResult{ StovePCResult::STOVE_PC_INVALID_PARAM };
    }

    std::string name = StringCast<ANSICHAR>(*Name).Get();
    std::string category1 = StringCast<ANSICHAR>(*Category1).Get();
    std::string category2 = StringCast<ANSICHAR>(*Category2).Get();
    float simpleValue = FCString::Atof(*SimpleValue.TrimQuotes());

    SDK_OBJECT_LOG("StovePC_StashCustomEvent --> Start");

    std::vector<StovePCCustomEventParameter> eventParams;
    for (int index = 0; index < ParamSize; index++)
    {
        StovePCCustomEventParameter param;
        param.name = new char[Params[index].Name.GetAllocatedSize()];
        memcpy(param.name, StringCast<ANSICHAR>(*Params[index].Name).Get(), Params[index].Name.GetAllocatedSize());
        param.value = new char[Params[index].Value.GetAllocatedSize()];
        memcpy(param.value, StringCast<ANSICHAR>(*Params[index].Value).Get(), Params[index].Value.GetAllocatedSize());

        eventParams.push_back(param);
    }

    StovePCResult result = GetStoveSDKModule().StashCustomEvent(name.c_str(), category1.c_str(), category2.c_str(), simpleValue, &eventParams[0], ParamSize);

    for (int index = 0; index < ParamSize; index++)
    {
        delete[] eventParams[index].name;
        delete[] eventParams[index].value;
    }
    eventParams.clear();

    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKSetLanguage(const FString& Language)
{
    std::string language = StringCast<ANSICHAR>(*Language).Get();
    StovePCResult result = GetStoveSDKModule().SetLanguage( language.c_str() );
    return FStoveResult{ result };
}

FString UStoveSDKObject::StoveSDKGetSDKVersion()
{
    return GetStoveSDKModule().GetSDKVersion();
}


void UStoveSDKObject::StoveSDKOpenUrl(const FString& Url)
{
    CHECK_POPUP_URL(Url);
    CHECK_MAKE_POPUP();

    FStovePopupControl PopupControl {
        FStovePopupControlUI {
                FStovePopupControlUIVisible {
                    true,
                    true,
                    true,
                    true,
                    true,
                    true,
                    true
                },
                1,
                FStovePopupControlUICloseButtonImage {
                    FStovePopupControlUICloseButtonImageDetail { "", "" },
                    FStovePopupControlUICloseButtonImageDetail { "", "" },
                    1
                    }
                }
    };

    _SetPopupAndDisplay(__func__, TArray<FPopupUIFrame>{ FPopupUIFrame(FString(""), FString(""), 0, Url, PopupControl) },
        TArray<FStovePopupRequestHeader>{},
        TArray<FStoveCefCookie>{});
}

void UStoveSDKObject::StoveSDKOpenAutoPopup(TArray<FStoveAutoPopup> AutoPopups, TArray<FStovePopupRequestHeader> Headers )
{
    CHECK_POPUP_LIST_EMPTY(AutoPopups);
    CHECK_HEADER_EMPTY();
    CHECK_MAKE_POPUP();

    TArray<FPopupUIFrame> PopupList;
    for (FStoveAutoPopup AutoPopup : AutoPopups)
    {
        PopupList.Push(FPopupUIFrame(AutoPopup.Origin, FString(""), AutoPopup.Id, AutoPopup.Url, AutoPopup.Control));
    }

    _SetPopupAndDisplay(__func__,
        PopupList,
        Headers,
        TArray<FStoveCefCookie>{});
}

void UStoveSDKObject::StoveSDKOpenManualPopup(TArray<FStoveManualPopup> ManualPopups, TArray<FStovePopupRequestHeader> Headers)
{
    CHECK_POPUP_LIST_EMPTY(ManualPopups);
    CHECK_HEADER_EMPTY();
    CHECK_MAKE_POPUP();

    TArray<FPopupUIFrame> PopupList;
    for (FStoveManualPopup ManualPopup : ManualPopups)
    {
        PopupList.Push(FPopupUIFrame{ ManualPopup.Origin, ManualPopup.ResourceKey, ManualPopup.Id, ManualPopup.Url, ManualPopup.Control });
    }

    _SetPopupAndDisplay(__func__,
        PopupList,
        Headers,
        TArray<FStoveCefCookie>{});
}

void UStoveSDKObject::StoveSDKOpenNewsPopup(const FStoveNewsPopup &NewsPopup, TArray<FStovePopupRequestHeader> Headers)
{
    CHECK_POPUP_URL(NewsPopup.Url);
    CHECK_HEADER_EMPTY();
    CHECK_MAKE_POPUP();

    _SetPopupAndDisplay(__func__,
        TArray<FPopupUIFrame>{ FPopupUIFrame{ NewsPopup.Origin, FString(""), NewsPopup.Id, NewsPopup.Url, NewsPopup.Control } },
        Headers,
        TArray<FStoveCefCookie>{});
}

void UStoveSDKObject::StoveSDKOpenCouponPopup(const FStoveCouponPopup &CouponPopup, TArray<FStovePopupRequestHeader> Headers)
{
    CHECK_POPUP_URL(CouponPopup.Url);
    CHECK_HEADER_EMPTY();
    CHECK_MAKE_POPUP();

    _SetPopupAndDisplay(__func__,
        TArray<FPopupUIFrame>{FPopupUIFrame{ FString(""), FString(""), 0, CouponPopup.Url, CouponPopup.Control } },
        Headers, TArray<FStoveCefCookie>{});
}

void UStoveSDKObject::StoveSDKOpenCommunityPopup(const FStoveCommunityPopup &CommunityPopup, TArray<FStovePopupRequestCookie> Cookies)
{
    CHECK_POPUP_URL(CommunityPopup.Url);
    CHECK_COOKIE_EMPTY();
    CHECK_MAKE_POPUP();

    auto ParseDomainArray = [](const FString& Url) -> TArray<FString> {
        TArray<FString> Array = {};
        TArray<FString> DomainParse = {};
        Url.ParseIntoArray(Array, TEXT("//"));

        auto fullDomain = FString(Array[1]);
        fullDomain.ParseIntoArray(DomainParse, TEXT("/"));

        return DomainParse;
    };

    auto ParsingUrl = ParseDomainArray(CommunityPopup.Url);

    FString UrlPath = "/";
    for (int index = 0; index < ParsingUrl.Num(); index++)
    {
        if (index == 0) continue;

        if (index + 1 == ParsingUrl.Num())
        {
            UrlPath += ParsingUrl[index];
        }
        else
        {
            UrlPath += ParsingUrl[index] + "/";
        }
    }


    TArray<FStoveCefCookie> StoveCookies;
    int index = 0;
    for (auto &Cookie : Cookies)
    {
        StoveCookies.Add(FStoveCefCookie{ ParsingUrl[0],
                                        UrlPath,
                                        Cookie.Name,
                                        Cookie.Value,
                                        false,
                                        false,
                                        0 });

        UE_LOG(LogStoveSDKPlugin, Log, TEXT("Cookie[%d] Domain : %s"), index, *ParsingUrl[0]);
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("Cookie[%d] UrlPath : %s"), index, *UrlPath);
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("Cookie[%d] Name : %s"), index, *Cookie.Name);
        UE_LOG(LogStoveSDKPlugin, Log, TEXT("Cookie[%d] Value : %s"), index, *Cookie.Value);
        index++;
    }

    _SetPopupAndDisplay(__func__,
        TArray<FPopupUIFrame>{ FPopupUIFrame{ FString(""), FString(""), 0, CommunityPopup.Url, CommunityPopup.Control } },
        TArray<FStovePopupRequestHeader>{},
        StoveCookies);
}


FStoveResult UStoveSDKObject::StoveSDKSetPopupDisallowed(const int PopupId, const int Days)
{
    StovePCResult result = GetStoveSDKModule().SetPopupDisallowed(PopupId, Days);
    return FStoveResult{ result };
}

const FString UStoveSDKObject::StoveSDKTranslateLanguage(const FString& StringId)
{
    std::string stringId = StringCast<ANSICHAR>(*StringId).Get();
    return GetStoveSDKModule().TranslateLanguage(stringId.c_str());
}

const FStoveTraceHint UStoveSDKObject::StoveSDKGetTraceHint()
{
    StovePCTraceHint traceHint = GetStoveSDKModule().GetTraceHint();

    FStoveTraceHint TraceHint {
        traceHint.result,
        traceHint.sessionId,
        traceHint.refSessionId,
        traceHint.uuid,
        traceHint.serviceProtocol,
        traceHint.refSourceType
    };

    return TraceHint;
}

FStoveResult UStoveSDKObject::StoveSDKFetchShopCategories()
{
    StovePCResult result = GetStoveSDKModule().FetchShopCategories();
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKFetchProducts(const FString& CategoryId, const bool IsRefresh)
{
    
    std::string categoryId = StringCast<ANSICHAR>(*CategoryId).Get();
    StovePCResult result = GetStoveSDKModule().FetchProducts(categoryId.c_str(), IsRefresh);
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKStartPurchase(const TArray<FStoveOrderProduct> Products, const int ProductSize)
{
    if (Products.Num() <= 0 ||
        Products.Num() != ProductSize ||
        ProductSize <= 0)
    {
        return FStoveResult{ StovePCResult::STOVE_PC_INVALID_PARAM };
    }

    SDK_OBJECT_LOG("StovePC_StartPurchase --> Start");

    std::vector<StovePCOrderProduct> orderProducts;
    for (int index = 0; index < ProductSize; index++)
    {
        StovePCOrderProduct orderProduct;
        orderProduct.productId = Products[index].ProductId;
        orderProduct.quantity = Products[index].Quantity;
        orderProduct.salePrice = Products[index].SalePrice;
        orderProducts.push_back(orderProduct);
    }

    StovePCResult result = GetStoveSDKModule().StartPurchase(&orderProducts[0], ProductSize);
    return FStoveResult{ result };
}

FStoveResult UStoveSDKObject::StoveSDKConfirmPurchase(const FString& TransactionMasterNo)
{
    int64 transactionMasterNo = FCString::Strtoi64(*TransactionMasterNo, NULL, 10);
    StovePCResult result = GetStoveSDKModule().ConfirmPurchase(transactionMasterNo);
    return FStoveResult{ result };
}

const FStovePurchaseVerification UStoveSDKObject::StoveSDKVerifyPurchase(const FString& CategoryId, const FString& ProductId)
{
    std::string categoryId = StringCast<ANSICHAR>(*CategoryId).Get();
    int64 productId = FCString::Strtoi64(*ProductId, NULL, 10);
    StovePCPurchaseVerification verify = GetStoveSDKModule().VerifyPurchase(categoryId.c_str(), productId);
    return FStovePurchaseVerification{ verify.result, verify.isPurchased};
}


FStoveResult UStoveSDKObject::StoveSDKFetchInventory()
{
    StovePCResult result = GetStoveSDKModule().FetchInventory();
    return FStoveResult{ result };
}

bool UStoveSDKObject::_MakePopupWidget(const FString &ApiName)
{
    if (_PopupWidget)
    {
        SDK_OBJECT_LOG_APINAME("%s: Only one popup API can be called at one time", ApiName);
        return false;
    }

    _PopupWidget = Cast<UStoveSDKPopup>(CreateWidget(GEngine->GameViewport->GetWorld(), UStoveSDKPopup::StaticClass()));

    if (!_PopupWidget)
    {
        SDK_OBJECT_LOG_APINAME("%s: Failed to create popup widget", ApiName);
        return false;
    }

    return true;
}

void UStoveSDKObject::_SetPopupAndDisplay(const FString &ApiName,
                                          TArray<FPopupUIFrame> Popups,
                                          const TArray<FStovePopupRequestHeader>& Headers,
                                          const TArray<FStoveCefCookie>& Cookies)
{
    _PopupWidget->Initiate(GetStoveSDKWeb());
    _PopupWidget->RegisterCloseCallback([this]() { _PopupWidget = nullptr; });
    _PopupWidget->SetPopupList(Popups);
    if (Headers.Num() > 0)
    {
        _PopupWidget->SetHeaders(Headers);
    }
    if (Cookies.Num() > 0)
    {
        _PopupWidget->SetCookies(Cookies);
    }
    _PopupWidget->Display();
    SDK_OBJECT_LOG_APINAME("%s: Success", ApiName);
}

void UStoveSDKObject::_SetPopupAndDisplayNoFrame(const FString &ApiName,
                                                TArray<FPopupUIFrame> Popups,
                                                const TArray<FStovePopupRequestHeader>& Headers,
                                                const TArray<FStoveCefCookie>& Cookies,
                                                const EStoveAlignment& Align)
{
    _PopupWidget->Initiate(GetStoveSDKWeb());
    _PopupWidget->RegisterCloseCallback([this]() { _PopupWidget = nullptr; });
    _PopupWidget->SetPopupList(Popups);
    _PopupWidget->SetAlignment(Align);
    _PopupWidget->SetNoFrame(true);
    if (Headers.Num() > 0)
    {
        _PopupWidget->SetHeaders(Headers);
    }
    if (Cookies.Num() > 0)
    {
        _PopupWidget->SetCookies(Cookies);
    }
    _PopupWidget->Display();
    SDK_OBJECT_LOG_APINAME("%s: Success", ApiName);
}
