﻿#pragma once

#include "CoreMinimal.h"
#include "Blueprint/UserWidget.h"
#include "Components/Button.h"
#include "StoveSDKWebInterface.h"
#include "StoveSDKStruct.h"
#include "StoveSDKEnum.h"
#include "Runtime/Core/Public/PixelFormat.h"
#include "Runtime/ImageWrapper/Public/IImageWrapper.h"
#include "Runtime/ImageWrapper/Public/IImageWrapperModule.h"
#include "Runtime/RenderCore/Public/RenderUtils.h"
#include "Runtime/Engine/Classes/Engine/Texture2D.h"
#include "StoveSDKPopup.generated.h"


USTRUCT(BlueprintType)
struct FPopupUIFrame
{
    GENERATED_USTRUCT_BODY()

    FPopupUIFrame()
    {}
    FPopupUIFrame(const FString &Origin,
        const FString &ResourceKey,
        const int Id,
        const FString &Url,
        const FStovePopupControl &Control,
        const bool UseLoadingAnimation = true)
        : Origin(Origin)
        , ResourceKey(ResourceKey)
        , Id(Id)
        , Url(Url)
        , bUseLoadingAnimation(UseLoadingAnimation)
    {
        bCloseButton = Control.UI.Visible.CloseButton;
        bNavigationBar = Control.UI.Visible.NavigationBar;
        bBackButton = Control.UI.Visible.BackButton;
        bForwardButton = Control.UI.Visible.ForwardButton;
        bRefreshButton = Control.UI.Visible.RefreshButton;
        bHomeButton = Control.UI.Visible.HomeButton;
        bDisallowedButton = Control.UI.Visible.DisallowedButton;
        Days = Control.UI.DisallowedDay;
        CloseNormalUrl = Control.UI.CloseButtonImage.Normal.FileUrl;
        CloseNormalId = Control.UI.CloseButtonImage.Normal.FileId;
        ClosePressedUrl = Control.UI.CloseButtonImage.Pressed.FileUrl;
        ClosePressedId = Control.UI.CloseButtonImage.Pressed.FileId;
        CloseType = Control.UI.CloseButtonImage.Type;
    }

    UPROPERTY()
        FStovePopupControl Control;

    UPROPERTY()
        FString Origin;

    UPROPERTY()
        FString ResourceKey;

    UPROPERTY()
        int Id;

    UPROPERTY()
        FString Url;

    UPROPERTY()
        bool bCloseButton;

    UPROPERTY()
        bool bNavigationBar;

    UPROPERTY()
        bool bBackButton;

    UPROPERTY()
        bool bForwardButton;

    UPROPERTY()
        bool bRefreshButton;

    UPROPERTY()
        bool bHomeButton;

    UPROPERTY()
        bool bDisallowedButton;

    UPROPERTY()
        bool bUseLoadingAnimation;

    UPROPERTY()
        int Days;

    UPROPERTY()
        FString CloseNormalUrl;

    UPROPERTY()
        FString CloseNormalId;

    UPROPERTY()
        FString ClosePressedUrl;

    UPROPERTY()
        FString ClosePressedId;

    UPROPERTY()
        int CloseType;
};

UCLASS()
class STOVESDKPLUGIN_API UStoveSDKPopup : public UUserWidget
{
    GENERATED_BODY()
public:
    UStoveSDKPopup(const FObjectInitializer &ObjectInitializer);

    typedef TFunction<void()> CloseCallback;

    void Initiate(TSharedPtr<IStoveSDKWebInterface> NewWebInterface);
    void SetPopupList(const TArray<FPopupUIFrame> &Popups);
    void EnableDimmedBackground(const bool IsEnable);
    void Display();

    void LoadURL();
    void SetHeaders(const TArray<FStovePopupRequestHeader> &Headers);
    void SetCookies(const TArray<FStoveCefCookie> &Cookies);
    void RegisterCloseCallback(const CloseCallback& OnCloseCallback);
    void SetAlignment(const EStoveAlignment& StoveAlignment);
    void SetNoFrame(const bool& NoFrame);
    void SetWidth(int32 FrameWidth) { _FrameWidth = FrameWidth; }

protected:
    virtual TSharedRef<SWidget> RebuildWidget() override;

private:
    void OnClickClose();
    void OnClickGoBack();
    void OnClickGoForward();
    void OnClickReload();
    void OnClickGoHome();
    void OnClickDisallowed();
    void SetDisallowedText();
    void SetDimmedBackground();
    void SetNewWebview();
    void SetPopupItemsVisibility();
    void SetCloseButtonStyle();
    void GetButtonTextureFromURL(FString &DefaultButtonURL, FString &PressedButtonURL);
    UTexture2D* MakeTexture(const TArray<uint8> &Content);
    void SetCloseButtonAsDefault();

    bool IsGoBackEnabled() const;
    bool IsGoForwardEnabled() const;
    bool IsReloadEnabled() const;
    bool IsHomeEnabled() const;

    float _NaviButtonSize;
    int32 _MarginDefaultSize;
    int32 _DisallowFontSize;

    TArray<FPopupUIFrame> _PopupList;

    int _PopupIndex;
    int _CloseButtonPresetCount;
    int _DefaultCloseButtonStyleType;
    bool _bUseDimmedBackground;
    bool _bIsNotFrame;
    TSharedPtr<SBox> _DimmedBackground;

    TArray<FStovePopupRequestHeader> _Headers;
    TArray<FStoveCefCookie> _Cookies;

    TSharedPtr<IStoveSDKWebInterface> _WebInterface;
    TSharedPtr<SWidget> _Frame;
    TSharedPtr<SWidget> _Webview;
    TSharedPtr<SBorder> _WebviewHolder;

    TSharedPtr<SBox> _NavigationBar;
    TSharedPtr<SBox> _CloseBox;
    TSharedPtr<SBox> _GoBackBox;
    TSharedPtr<SBox> _GoForwardBox;
    TSharedPtr<SBox> _ReloadBox;
    TSharedPtr<SBox> _HomeBox;
    TSharedPtr<SBox> _DisallowedBox;
    TSharedPtr<STextBlock> _DisallowedText;
    FMargin _ButtonPressedPadding;

    TArray<FButtonStyle*> _CloseButtonStyles;
    TSharedPtr<SButton> _CloseButton;
    FButtonStyle* _GoBackButtonStyle;
    FButtonStyle* _GoForwardButtonStyle;
    FButtonStyle* _ReloadButtonStyle;
    FButtonStyle* _HomeButtonStyle;
    FButtonStyle* _DisallowedButtonStyle;
    CloseCallback _OnClose;
    IImageWrapperModule* _ImageWrapperModule;
    TFuture<UTexture2D*> _CloseButtonTextureDefaultFromWeb;
    TFuture<UTexture2D*> _CloseButtonTexturePressedFromWeb;
    EVerticalAlignment _VertialAlign;
    EHorizontalAlignment _HorizontalAlign;
    
    int32 _FrameWidth;
};
